module butter_filt 
implicit none
  real,private,save :: pi
!conversion of Dave Hale's Butterworth stuff
!CWP
  contains
!Butterworth filter:  compute number of poles and -3 db frequency
!for a low-pass or high-pass filter, given a frequency response
!constrained at two frequencies.
!******************************************************************************
!Input:
!fpass   frequency in pass band at which amplitude is >= apass
!apass   amplitude in pass band corresponding to frequency fpass
!fstop     frequency in stop band at which amplitude is <= astop
!astop   amplitude in stop band corresponding to frequency fstop
!
!Output:
!npoles    number of poles
!f3db    frequency at which amplitude is sqrt(0.5) (-3 db)
!
  subroutine bfdesign(fpass,apass,fstop,astop,npoles,f3db)
    real :: fpass,apass,fstop,astop
    integer :: npoles
    real   :: f3db
    real :: wpass,wstop,fnpoles,w3db
    pi=acos(-1.)
    PI = acos(-1.)
    wpass = 2.0*tan(PI*fpass)
    wstop = 2.0*tan(PI*fstop)
    if (fstop>fpass) then
      fnpoles = log((1.0/(apass*apass)-1.0)/(1.0/(astop*astop)-1.0))  / log((wpass/wstop)**2.0&
        &)
      w3db = wpass/((1.0/(apass*apass)-1.0)**(0.5/fnpoles))
!   else, if highpass filter, then
    else
      fnpoles = log((1.0/(apass*apass)-1.0)/(1.0/(astop*astop)-1.0))  / log((wstop/wpass)**2&
        &)
      w3db = wpass*((1.0/(apass*apass)-1.0)**0.5/fnpoles)
    end if 
!   determine integer number of poles
    npoles = 1+floor(fnpoles)
!   determine (unwarped) -3 db frequency
    f3db = atan(0.5*w3db)/PI
  end subroutine 
  subroutine bfhighpass(npoles,f3db,p,q)
    integer :: jpair,j,k,n
    real :: f3db
    real :: p(:),q(:)
    integer :: npoles
    real    ::  r,scale,theta,a,b1,b2,pj,pjm1,pjm2,qjm1,qjm2
    n=size(p)
    r = 2.0*tan(PI*abs(f3db))
    if (mod(npoles,2).ne.0) then
      scale = r+2.0
      a = 2.0/scale
      b1 = (r-2.0)/scale
      pj = 0.0
      qjm1 = 0.0
      do j=1,n 
        pjm1 = pj
        pj = p(j)
        q(j) = a*(pj-pjm1)-b1*qjm1
        qjm1 = q(j)
      end do
    else
      q=p 
    end if 
    do k=1,npoles/2 
      jpair=k-1
      theta = PI*(2*jpair+1)/(2*npoles)
      scale = 4.0+4.0*r*sin(theta)+r*r
      a = 4.0/scale
      b1 = (2.0*r*r-8.0)/scale
      b2 = (4.0-4.0*r*sin(theta)+r*r)/scale
      pjm1 = 0.0
      pj = 0.0
      qjm2 = 0.0
      qjm1 = 0.0
      do j=1,n 
        pjm2 = pjm1
        pjm1 = pj
        pj = q(j)
        q(j) = a*(pj-2.0*pjm1+pjm2)-b1*qjm1-b2*qjm2
        qjm2 = qjm1
        qjm1 = q(j)
      end do
    end do
  end subroutine 
  subroutine bflowpass(npoles,f3db,p,q)
    real  :: p(:),q(:)
    real  :: f3db
    integer   :: npoles
    integer :: jpair,j,k,n
    real  r,scale,theta,a,b1,b2,pj,pjm1,pjm2,qjm1,qjm2
    n=size(p)
    r = 2.0*tan(PI*abs(f3db))
    if (mod(npoles,2).ne.0) then
      scale = r+2.0
      a = r/scale
      b1 = (r-2.0)/scale
      pj = 0.0
      qjm1 = 0.0
      do j=1,n 
        pjm1 = pj
        pj = p(j)
        q(j) = a*(pj+pjm1)-b1*qjm1
        qjm1 = q(j)
      end do
    else
      q=p
    end if 
    do k=1,npoles/2 
      jpair=k-1
      theta = PI*(2*jpair+1)/(2*npoles)
      scale = 4.0+4.0*r*sin(theta)+r*r
      a = r*r/scale
      b1 = (2.0*r*r-8.0)/scale
      b2 = (4.0-4.0*r*sin(theta)+r*r)/scale
      pjm1 = 0.0
      pj = 0.0
      qjm2 = 0.0
      qjm1 = 0.0
      do j=1,n 
        pjm2 = pjm1
        pjm1 = pj
        pj = q(j)
        q(j) = a*(pj+2.0*pjm1+pjm2)-b1*qjm1-b2*qjm2
        qjm2 = qjm1
        qjm1 = q(j)
      end do
    end do
  end subroutine 
end module 
